#ifndef _LINUX_DMSDOS_FS_CONFIG_H
#define _LINUX_DMSDOS_FS_CONFIG_H

/*
 * The DMSDOS filesystem -- user configurable defines
 */

/**************************************************************************

   Be sure to know what you are doing if you change something herein. Some
   parameters have hard-coded limitations (e.g. maximum values, or must be
   a power of 2), please don't break them.

   Some of the switches may have a significant effect on system performance.
   I haven't optimized the parameters for speed yet; some are based on
   calculations, others on assumptions, and others are simply guessed. So
   if you play with the parameters and you manage to get a great win in
   performance, please let me know. Please don't forget to report also the
   amount of RAM in your system since the optimum settings seem to depend
   heavily on it.

**************************************************************************/

/* define this for kernel 2.1.76 or newer */
#define __FOR_KERNEL_2_1_76

/* define this for kernel 2.1.78 or newer */
#define __FOR_KERNEL_2_1_78

/* if defined, use vmalloc instead of kmalloc */
/* This is to fix 'no memory' messages that massively occured in older 
   dmsdos versions. kmalloc is faster but more likely to fail (i.e. claim
   that the system is out of memory) - vmalloc very rarely fails but memory
   is swappable and it is surely not a good idea to swap the disk cache to the
   disk... vmalloc is just the safe solution. If you have a system with lots
   of physical RAM you might want to try kmalloc for better performance. */
/* this define has only an effect if USE_XMALLOC is undefined */
/* default value: defined, recommended value: neither, see USE_XMALLOC */
#define USE_VMALLOC

/* optimized memory allocation routine - it decides itself whether to
   use kmalloc or vmalloc depending on the requested size and on what the
   system can provide */
/* you may also want to define MAX_KMALLOC_SIZE to something reasonable */
/* if defined, this overrides USE_VMALLOC above */
/* WARNING: this is experimental code - if you get a message saying
   "please disable XMALLOC" it is broken in your kernel */
/* you may receive *lots* of "Couldn't get a free page" messages in the syslog
   which can be safely ignored - they result from the driver's memory 
   allocation strategy */
/* WARNING: DON'T USE XMALLOC WHEN DEBUGGING NEW CODE - USE VMALLOC
   because vmalloc'd memory causes immediate segfaults on illegal access */
/* default value: defined, recommended value: defined */
/*#define USE_XMALLOC*/
/*#define MAX_KMALLOC_SIZE what_you_think_is_good*/ 
/* there's a default of 128KB defined in dmsdos_alloc.c if you don't define 
   it here - the kernel has also a built-in limit depending on your system's
   PAGE_SIZE, see kernel/kmalloc.c */

/* maximum number of clusters that can be delayed for compression -
   this is for the dmsdos compression daemon / not for write-back caching.
   A value larger than 4096 seems to slow down your system more than
   not using the daemon since the driver is busy with maintaining the cache
   most of the time. Still good performance: 1024 */
/* default value: 1024, recommended value: unknown */
#define LISTSIZE 1024 /* must be a power of 2 */

/* time (in seconds) after which free_idle_cache returns idle buffers to
   the kernel -- which in turn decides what to do with them -- this is
   for mdfat/fat/bitfat caching */
/* default value: 60, recommended value: unknown */
#define MAX_CACHE_TIME 60

/* number of cvfs that can be handled at the same time */
/* the following number *must* stay below 31 */
/* default value: 10, recommended value: number of your compressed drives */
#define MAXDBLFILES 10

/* mdfat/fat/bitfat cache sizes -- please note that the sum of these three
   are the number of fs buffers dmsdos *permanently* uses - keep small on
   systems that are short on memory */
/* default values: 40;20;10, recommended values: unknown */
#define MDFATCACHESIZE 40
#define DFATCACHESIZE 20
#define BITFATCACHESIZE 10

/* change the 'define' to 'undef' if you want to inhibit write access 
   by _not_ _compiling_ _in_ the low-level write code */
/* default value: defined, recommended value: defined */
/*#define DBL_WRITEACCESS*/

/* debug logging */
/* causes *lots* of debug messages in syslog if !=0 - in this case you are 
   strongly encouraged to increase the kernel log buffer size to at least 64KB
   (in file kernel/printk.c: define LOG_BUF_LEN to 65536 instead of 4096 or 
   8192 - but note that the LOG_BUF_LEN value *must* be a power of 2). */
/* default value: 0, recommended value: unless you are debugging: 0 */
/* loglevel can be changed at run time with 'dutil . setloglevel xyz' */
#define DEFAULT_LOGLEVEL 0

/* GUESS_HACK forces the compression scheme guessing code to return JM-0-1 as
   result when it in fact has guessed SQ-0-0. This is because SQ-0-0 is
   currently unsupported, so write access would be always uncompressed on
   SQ-0-0 compressed drives. 'undef' this to avoid this hack */ 
/* default value: defined, recommended value: defined */
#define GUESS_HACK

/* default compression mode - you should use READ_ONLY or GUESS */
/* default value: GUESS, recommended value: GUESS */
/* compression mode can be changed at run time with 'dutil . setcomp xyz' */
#define DEFAULT_COMP GUESS

/* default compression level minus 1 */
/* 0=least efficient but fastest compression
   11=most efficient but slowest compression */
/* For slow processors you may want to use a lower value */
/* default value: 11, recommended value: depends on your cpu speed and what 
   slowdown you can tolerate */
/* compression level can be changed at run time with 'dutil . setcf xyz' */
#define DEFAULT_CF   11

/* maximum number of clusters in cluster cache */
/* keep small on systems with low memory - otherwise dmsdos may eat up too
   much memory and slow down your system by causing great swapping -
   every cluster uses 8KB (32KB on drivespace 3)
   you shouldn't exceed approx. 1/4 of your physical ram
   so stay below 64 if you have only 4MB
   stay below 64 if you have only 16MB and you are using drivespace 3 
   If this value is too low, applications using files on a compressed
   partition lock out each other. If it is much too low, they may even lock
   out themselves and thus lock up the whole system. Values below 30 are
   supposed critical, values below 10 are supposed dangerous. */
/* default value: 64, recommended value: unknown */
#define CCACHESIZE 64

/* time in seconds after which unused clusters are removed from the cluster 
   cache when free_idle_ccache is called */
/* default value: 240, recommended value: unknown */
#define MAX_CCACHE_TIME 240

/* whether to fill intermediate clusters on write access... supposed to be
   obsolete... define it if your files become unreadable under dos...
   and *please* send a problem report in that case */
/* default value: undefined, recommended value: undefined */
#undef FILLWRITE

/* Maximum number of sectors to read-ahead at once - default is 64 in order 
   to let a drivespace 3 cluster fit in */
/* this parameter has only an effect when read-ahead is enabled */
/* default value: 64, recommended value: 64 */
#define MAX_READA 64

/* Use experimental read-ahead code in order to bypass wait_on_buffer ? */
/* If undefined, the standard kernel read-ahead code is used */
/* default value: undefined, recommended value: unknown */
#define USE_READA_LIST

/* this is only used if USE_READA_LIST is defined */
/* selects the number of read-aheads that can be done without calling 
   wait_on_buffer */
/* works by cyclic buffering of the buffer_heads -  so wait_on_buffer is
   called READA_LIST_SIZE read-aheads later than without USE_READA_LIST. 
   Hopefully then waiting is no longer necessary...*/
/* my own benchmarks didn't lead to a clear result :-/ */
/* default value: 256, recommended value: unknown */
#define READA_LIST_SIZE 256

/* Read-ahead threshold - read-ahead is only done when more than this amount
   of bytes are read at once. This prevents a badly written program from
   issuing thousands of useless read-aheads. 4095 is a good value since this
   allows read-ahead just for a memory page (mmap gets the data in 4096 byte
   blocks and should have a gain from read-ahead). Lower values may cause 
   clustersize/(value+1)-1 useless read-aheads per cluster. */
/* default value: 4095, recommended value: 4095 */
#define READA_THRESHOLD 4095

/* Generic speedup tricks - it's a nice collection. Some more ideas?
   Bit coded speedup field, speedup is selected if the appropriate bit is set.
*/
#define SP_NO_DIR_COMPR 0x0001
/* bit #0: Never compress directories for drivespace 3 and stacker (others
           don't support compressed directories at all) [recommended]. 
           Directories are accessed very often and otherwise had to be
           decompressed and compressed again after each access.
*/
#define SP_NO_EMD_COMPR 0x0002
/* bit #1: For umsdos upon dmsdos: never compress the --linux-.--- special
           file (it contains extended directory information) [strongly 
           recommended]. This file is even more often written than a directory
           since it contains the access times (the directory contains only
           the modification times). You will regret compressing this file,
           even if you own the fastest computer of the world.
*/
#define SP_NO_EXACT_SEARCH 0x0004
/*   bit #2: Search faster but less carefully for free space in bitfat at the
           cost of more fragmentation [not recommended]. This bit is for 
           doublespace/drivespace sector allocation code. If you set this bit,
           allocation on large CVFs is faster but also cause a little more
           fragmentation. Searching carefully leads to sudden system freezes
           for a second or so on large partitions. Use this switch if you 
           are annoyed by them.
*/
#define SP_NO_UNMOUNT_COMPR 0x0008
/* bit #3: Write dirty buffers on unmount immediately without compressing them
           before [recommended]. This bit defines what happens with unwritten 
           dirty clusters that are in the cache when the filesystem is 
           unmounted. If the bit is cleared it means write them back, but
           compress them before. If it is set it means write them back without
           compression.  *** COMPRESSION TAKES SOME TIME, YOU'LL NOTICE
           IT *** Since most people who haven't read these lines tend to press
           the reset button when a filesystem unmount takes some minutes 
           (especially on a shutdown/reboot), I've set this bit. So the 
           unmount is fast and the users don't panic. :)
*/
#define SP_USE_WRITE_BACK 0x0010
/* bit #4: Enable write-back cluster caching [recommended]. If this bit is set
           the data in the cluster cache that have been changed by an 
           application and have to be written back to disk are not written
           back immediately - they are kept some time in memory just in order
           to save disk access when the application again changes the data.
           There's a small risk of data loss if the system happens to crash
           before the data are written back - but since your operating system
           is Linux a crash is rather unlikely :)
*/
#define SP_USE_READ_AHEAD 0x0020
/* bit #5: Enable cluster read-ahead [recommended]. If set, this bit causes
           the driver to initiate a low-level disk access for some data when 
           it thinks they are most likely needed in a short time later. The 
           trick is that the driver doesn't wait for the disk access to 
           finish. So the disk can position its head (and so on) while the 
           system can do something else. Most likely the disk access has 
           finished when the data are actually needed - this saves some time 
           we otherwise had to wait. Well, at least this is the nice idea 
           of read-ahead. However, since read-ahead relies on a prediction, 
           there may be situations when it is useless or even a loss.
*/
#define SP_FAST_BITFAT_ALLOC 0x0040
/* bit #6: Switch to very fast sector allocation strategy [not recommended].
           This speeds up bitfat allocation by using heuristics instead of
           a well-calculated algorithm... BE WARNED, it causes *much*
           fragmentation in very short time. The "filesystem almost full"
           warning is suppressed (because it's meaningless with heuristics).
           This switch is meant as a last resort if you cannot tolerate
           system slowdowns at all. Don't fill the compressed filesystem
           up to more than 3/4 when this switch is set. Write access may 
           fail horribly and cause data loss due to too high fragmentation.
           (Well, this is Dos/Win95 strategy - never tell the disadvantages)
*/
#define SP_USE_DAEMON 0x0080
/* bit #7: use the daemon for delayed compression in the background
           [recommended]. This speeds up write access a lot, but may cause
           undesired side effects (see documentation for details).*/
/* default value: 187(=1+2+0+8+16+32+128), recommended value: see above in [] */
/* speedup flags can be changed at run time with 'dutil . setspeedup xyz' */
#define DEFAULT_SPEEDUP 187

/* This selects whether to use the internal dmsdos daemon or not.
   When the internal daemon is used, the external dmsdosd won't run
   though you can compile it - it always exits with a "no permission" 
   error message.
   The internal daemon compresses faster (because the data needn't be copied
   from kernel space to user space and back), but seems to eat up much system 
   time - use it only on *really fast* machines. The external daemon eats up 
   minimal system time and ressources. In a future version, the external
   daemon may even be able to cooperate with stacker's compression routines 
   in stacker.bin (by calling them - you need to own a copy of stacker for
   this purpose).*/
/* default value: undefined, recommended value: unknown */
#undef INTERNAL_DAEMON

/* Time interval in seconds for internal daemon -
   it awakes periodically and looks for data or idle cache in this interval */
#define IDMSDOSD_TIME 30

/* define this if you need a really small kernel - this leaves out the
   logging messages - untested, not recommended at all */
#undef NOLOG

/* define this to generate a sequence number for each log line. The numbers
   are guaranteed to be uniqe and in order of call. */
#define SEQLOG

/* define this to use new readpage interface (for writable mmap)
   undefine this to use old mmap interface which is read-only */
#define DMSDOS_USE_READPAGE

#endif
